# Generated by Django 3.2.11 on 2022-01-19 04:59

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import djstripe.enums
import djstripe.fields


class Migration(migrations.Migration):
    dependencies = [
        ("djstripe", "0010_alter_customer_balance"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="subscriptionschedule",
            name="billing_thresholds",
        ),
        migrations.AddField(
            model_name="webhookeventtrigger",
            name="webhook_endpoint",
            field=djstripe.fields.StripeForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="djstripe.webhookendpoint",
                to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                help_text="The endpoint this webhook was received on",
            ),
        ),
        migrations.AddField(
            model_name="account",
            name="djstripe_owner_account",
            field=djstripe.fields.StripeForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="djstripe.account",
                to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                help_text="The Stripe Account this object belongs to.",
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="api_version",
            field=models.CharField(
                blank=True,
                help_text=(
                    "the API version at which the event data was rendered. Blank for"
                    " old entries only, all new entries will have this value"
                ),
                max_length=64,
            ),
        ),
        migrations.AlterField(
            model_name="webhookendpoint",
            name="api_version",
            field=models.CharField(
                max_length=64,
                blank=True,
                help_text=(
                    "The API version events are rendered as for this webhook endpoint."
                    " Defaults to the configured Stripe API Version."
                ),
            ),
        ),
        migrations.AddField(
            model_name="coupon",
            name="applies_to",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="subscriptionschedule",
            name="subscription",
            field=models.ForeignKey(
                blank=True,
                help_text=(
                    "ID of the subscription managed by the subscription schedule."
                ),
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="subscriptions",
                to="djstripe.subscription",
            ),
        ),
        migrations.AddField(
            model_name="taxrate",
            name="country",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Two-letter country code.",
                max_length=2,
            ),
        ),
        migrations.AddField(
            model_name="taxrate",
            name="state",
            field=models.CharField(
                blank=True,
                default="",
                help_text="ISO 3166-2 subdivision code, without country prefix.",
                max_length=2,
            ),
        ),
        migrations.AddField(
            model_name="taxrate",
            name="tax_type",
            field=models.CharField(
                blank=True,
                default="",
                help_text=(
                    "The high-level tax type, such as vat, gst, sales_tax or custom."
                ),
                max_length=50,
            ),
        ),
        migrations.AlterField(
            model_name="payout",
            name="failure_code",
            field=djstripe.fields.StripeEnumField(
                blank=True,
                default="",
                enum=djstripe.enums.PayoutFailureCode,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="billing_reason",
            field=djstripe.fields.StripeEnumField(
                blank=True,
                default="",
                enum=djstripe.enums.InvoiceBillingReason,
                max_length=38,
            ),
        ),
        migrations.AlterField(
            model_name="upcominginvoice",
            name="billing_reason",
            field=djstripe.fields.StripeEnumField(
                blank=True,
                default="",
                enum=djstripe.enums.InvoiceBillingReason,
                max_length=38,
            ),
        ),
        migrations.AddField(
            model_name="subscriptionitem",
            name="proration_behavior",
            field=djstripe.fields.StripeEnumField(
                blank=True,
                default="create_prorations",
                enum=djstripe.enums.SubscriptionProrationBehavior,
                help_text=(
                    "Determines how to handle prorations when the billing cycle changes"
                    " (e.g., when switching plans, resetting billing_cycle_anchor=now,"
                    " or starting a trial), or if an item’s quantity changes"
                ),
                max_length=17,
            ),
        ),
        migrations.AddField(
            model_name="subscriptionitem",
            name="proration_date",
            field=djstripe.fields.StripeDateTimeField(
                blank=True,
                help_text=(
                    "If set, the proration will be calculated as though the"
                    " subscription was updated at the given time. This can be used to"
                    " apply exactly the same proration that was previewed with upcoming"
                    " invoice endpoint. It can also be used to implement custom"
                    " proration logic, such as prorating by day instead of by second,"
                    " by providing the time that you wish to use for proration"
                    " calculations"
                ),
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="subscription",
            name="proration_behavior",
            field=djstripe.fields.StripeEnumField(
                blank=True,
                default="create_prorations",
                enum=djstripe.enums.SubscriptionProrationBehavior,
                help_text=(
                    "Determines how to handle prorations when the billing cycle changes"
                    " (e.g., when switching plans, resetting billing_cycle_anchor=now,"
                    " or starting a trial), or if an item’s quantity changes"
                ),
                max_length=17,
            ),
        ),
        migrations.AddField(
            model_name="subscription",
            name="proration_date",
            field=djstripe.fields.StripeDateTimeField(
                blank=True,
                help_text=(
                    "If set, the proration will be calculated as though the"
                    " subscription was updated at the given time. This can be used to"
                    " apply exactly the same proration that was previewed with upcoming"
                    " invoice endpoint. It can also be used to implement custom"
                    " proration logic, such as prorating by day instead of by second,"
                    " by providing the time that you wish to use for proration"
                    " calculations"
                ),
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="subscription",
            name="pause_collection",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.CreateModel(
            name="TaxCode",
            fields=[
                ("djstripe_created", models.DateTimeField(auto_now_add=True)),
                ("djstripe_updated", models.DateTimeField(auto_now=True)),
                (
                    "djstripe_id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("id", djstripe.fields.StripeIdField(max_length=255, unique=True)),
                (
                    "livemode",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text=(
                            "Null here indicates that the livemode status is unknown or"
                            " was previously unrecorded. Otherwise, this field"
                            " indicates whether this record comes from Stripe test mode"
                            " or live mode operation."
                        ),
                        null=True,
                    ),
                ),
                ("created", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                (
                    "description",
                    models.TextField(
                        blank=True, help_text="A description of this object.", null=True
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="A short name for the tax code.", max_length=128
                    ),
                ),
                (
                    "djstripe_owner_account",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.account",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                        help_text="The Stripe Account this object belongs to.",
                    ),
                ),
            ],
            options={
                "get_latest_by": "created",
                "verbose_name": "Tax Code",
            },
        ),
        migrations.CreateModel(
            name="Order",
            fields=[
                ("djstripe_created", models.DateTimeField(auto_now_add=True)),
                ("djstripe_updated", models.DateTimeField(auto_now=True)),
                (
                    "djstripe_id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("id", djstripe.fields.StripeIdField(max_length=255, unique=True)),
                (
                    "livemode",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text=(
                            "Null here indicates that the livemode status is unknown or"
                            " was previously unrecorded. Otherwise, this field"
                            " indicates whether this record comes from Stripe test mode"
                            " or live mode operation."
                        ),
                        null=True,
                    ),
                ),
                ("created", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                ("metadata", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "description",
                    models.TextField(
                        blank=True, help_text="A description of this object.", null=True
                    ),
                ),
                ("amount_subtotal", djstripe.fields.StripeQuantumCurrencyAmountField()),
                ("amount_total", djstripe.fields.StripeQuantumCurrencyAmountField()),
                (
                    "application",
                    models.CharField(
                        blank=True,
                        help_text=(
                            "ID of the Connect application that created the Order, if"
                            " any."
                        ),
                        max_length=255,
                    ),
                ),
                ("automatic_tax", djstripe.fields.JSONField()),
                ("billing_details", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "client_secret",
                    models.TextField(
                        help_text=(
                            "The client secret of this PaymentIntent. Used for"
                            " client-side retrieval using a publishable key."
                        ),
                        max_length=5000,
                    ),
                ),
                ("currency", djstripe.fields.StripeCurrencyCodeField(max_length=3)),
                ("discounts", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "ip_address",
                    models.GenericIPAddressField(
                        blank=True,
                        help_text=(
                            "A recent IP address of the purchaser used for tax"
                            " reporting and tax location inference."
                        ),
                        null=True,
                    ),
                ),
                ("line_items", djstripe.fields.JSONField()),
                ("payment", djstripe.fields.JSONField()),
                ("shipping_cost", djstripe.fields.JSONField(blank=True, null=True)),
                ("shipping_details", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "status",
                    djstripe.fields.StripeEnumField(
                        enum=djstripe.enums.OrderStatus, max_length=10
                    ),
                ),
                ("tax_details", djstripe.fields.JSONField(blank=True, null=True)),
                ("total_details", djstripe.fields.JSONField()),
                (
                    "customer",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="djstripe.customer",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                        help_text="The customer which this orders belongs to.",
                    ),
                ),
                (
                    "djstripe_owner_account",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.account",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                        help_text="The Stripe Account this object belongs to.",
                    ),
                ),
                (
                    "payment_intent",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="djstripe.paymentintent",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                        help_text=(
                            "ID of the payment intent associated with this order. Null"
                            " when the order is open."
                        ),
                    ),
                ),
            ],
            options={
                "get_latest_by": "created",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ShippingRate",
            fields=[
                ("djstripe_created", models.DateTimeField(auto_now_add=True)),
                ("djstripe_updated", models.DateTimeField(auto_now=True)),
                (
                    "djstripe_id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("id", djstripe.fields.StripeIdField(max_length=255, unique=True)),
                (
                    "livemode",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text=(
                            "Null here indicates that the livemode status is unknown or"
                            " was previously unrecorded. Otherwise, this field"
                            " indicates whether this record comes from Stripe test mode"
                            " or live mode operation."
                        ),
                        null=True,
                    ),
                ),
                ("created", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                ("metadata", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "active",
                    models.BooleanField(
                        default=True,
                        help_text=(
                            "Whether the shipping rate can be used for new purchases."
                            " Defaults to true"
                        ),
                    ),
                ),
                (
                    "display_name",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text=(
                            "The name of the shipping rate, meant to be displayable to"
                            " the customer. This will appear on CheckoutSessions."
                        ),
                        max_length=50,
                    ),
                ),
                ("fixed_amount", djstripe.fields.JSONField()),
                (
                    "type",
                    djstripe.fields.StripeEnumField(
                        default="fixed_amount",
                        enum=djstripe.enums.ShippingRateType,
                        max_length=12,
                    ),
                ),
                ("delivery_estimate", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "tax_behavior",
                    djstripe.fields.StripeEnumField(
                        enum=djstripe.enums.ShippingRateTaxBehavior, max_length=11
                    ),
                ),
                (
                    "djstripe_owner_account",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.account",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                        help_text="The Stripe Account this object belongs to.",
                    ),
                ),
                (
                    "tax_code",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.taxcode",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                        help_text="The shipping tax code",
                    ),
                ),
            ],
            options={
                "get_latest_by": "created",
                "verbose_name": "Shipping Rate",
            },
        ),
    ]



"""
Django Administration Custom Filters Module
"""
from django.contrib import admin

from djstripe import models


class BaseHasSourceListFilter(admin.SimpleListFilter):
    title = "source presence"
    parameter_name = "has_source"

    def lookups(self, request, model_admin):
        """
        Return a list of tuples.

        The first element in each tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        source:
        https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        return (("yes", "Has a source"), ("no", "Has no source"))

    def queryset(self, request, queryset):
        """
        Return the filtered queryset based on the value provided in the query string.

        source:
        https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        filter_args = {self._filter_arg_key: None}

        if self.value() == "yes":
            return queryset.exclude(**filter_args)
        if self.value() == "no":
            return queryset.filter(**filter_args)


class CustomerHasSourceListFilter(BaseHasSourceListFilter):
    _filter_arg_key = "default_source"


class InvoiceCustomerHasSourceListFilter(BaseHasSourceListFilter):
    _filter_arg_key = "customer__default_source"


class CustomerSubscriptionStatusListFilter(admin.SimpleListFilter):
    """A SimpleListFilter used with Customer admin."""

    title = "subscription status"
    parameter_name = "sub_status"

    def lookups(self, request, model_admin):
        """
        Return a list of tuples.

        The first element in each tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        source:
        https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        statuses = [
            [x, x.replace("_", " ").title()]
            for x in models.Subscription.objects.values_list(
                "status", flat=True
            ).distinct()
        ]
        statuses.append(["none", "No Subscription"])
        return statuses

    def queryset(self, request, queryset):
        """
        Return the filtered queryset based on the value provided in the query string.

        source:
        https://docs.djangoproject.com/en/1.10/ref/contrib/admin/#django.contrib.admin.ModelAdmin.list_filter
        """
        if self.value() is None:
            return queryset.all()
        else:
            return queryset.filter(subscriptions__status=self.value()).distinct()



import stripe
from django.db import models

from djstripe.models.billing import Discount
from djstripe.settings import djstripe_settings

from ..enums import OrderStatus
from ..fields import (
    JSONField,
    StripeCurrencyCodeField,
    StripeEnumField,
    StripeForeignKey,
    StripeQuantumCurrencyAmountField,
)
from ..settings import djstripe_settings
from .base import StripeModel


class Order(StripeModel):
    """
    An Order describes a purchase being made by a customer,
    including the products & quantities being purchased, the order status,
    the payment information, and the billing/shipping details.

    Stripe documentation: https://stripe.com/docs/api/orders_v2/object?lang=python
    """

    stripe_class = stripe.Order
    expand_fields = ["customer", "line_items", "discounts", "total_details.breakdown"]
    stripe_dashboard_item_name = "orders"

    amount_subtotal = StripeQuantumCurrencyAmountField(
        help_text=(
            "Order cost before any discounts or taxes are applied. A positive integer"
            " representing the subtotal of the order in the smallest currency unit"
            " (e.g., 100 cents to charge $1.00 or 100 to charge ¥100, a zero-decimal"
            " currency)."
        )
    )
    amount_total = StripeQuantumCurrencyAmountField(
        help_text=(
            "Total order cost after discounts and taxes are applied. A positive integer"
            " representing the cost of the order in the smallest currency unit (e.g.,"
            " 100 cents to charge $1.00 or 100 to charge ¥100, a zero-decimal"
            " currency). To submit an order, the total must be either 0 or at least"
            " $0.50 USD or equivalent in charge currency."
        )
    )
    application = models.CharField(
        max_length=255,
        blank=True,
        help_text="ID of the Connect application that created the Order, if any.",
    )
    automatic_tax = JSONField(
        help_text="Settings and latest results for automatic tax lookup for this Order."
    )
    billing_details = JSONField(
        null=True,
        blank=True,
        help_text="Customer billing details associated with the order.",
    )
    client_secret = models.TextField(
        max_length=5000,
        help_text=(
            "The client secret of this PaymentIntent. "
            "Used for client-side retrieval using a publishable key."
        ),
    )
    currency = StripeCurrencyCodeField(
        help_text=(
            "Three-letter ISO currency code, in lowercase. Must be a supported"
            " currency."
        )
    )
    # not deleting order when customer is deleted, because order may be important for taxation and audit purposes
    customer = StripeForeignKey(
        "Customer",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        help_text="The customer which this orders belongs to.",
    )
    discounts = JSONField(
        null=True,
        blank=True,
        help_text="The discounts applied to the order.",
    )
    ip_address = models.GenericIPAddressField(
        null=True,
        blank=True,
        help_text=(
            "A recent IP address of the purchaser used for tax reporting and tax"
            " location inference."
        ),
    )
    line_items = JSONField(
        help_text=(
            "A list of line items the customer is ordering. Each line item includes"
            " information about the product, the quantity, and the resulting cost."
            " There is a maximum of 100 line items."
        ),
    )
    payment = JSONField(
        help_text=(
            "Payment information associated with the order. Includes payment status,"
            " settings, and a PaymentIntent ID"
        ),
    )
    payment_intent = StripeForeignKey(
        "PaymentIntent",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        help_text=(
            "ID of the payment intent associated with this order. Null when the order"
            " is open."
        ),
    )
    shipping_cost = JSONField(
        null=True,
        blank=True,
        help_text=(
            "The details of the customer cost of shipping, including the customer"
            " chosen ShippingRate."
        ),
    )
    shipping_details = JSONField(
        null=True,
        blank=True,
        help_text="Customer shipping information associated with the order.",
    )
    status = StripeEnumField(
        enum=OrderStatus, help_text="The overall status of the order."
    )
    tax_details = JSONField(
        null=True,
        blank=True,
        help_text="Tax details about the purchaser for this order.",
    )
    total_details = JSONField(
        help_text=(
            "Tax, discount, and shipping details for the computed total amount of this"
            " order."
        ),
    )

    def __str__(self):
        template = f"on {self.created.strftime('%m/%d/%Y')} ({self.status})"
        if self.status in (OrderStatus.open, OrderStatus.canceled):
            return "Created " + template
        elif self.status in (
            OrderStatus.submitted,
            OrderStatus.complete,
            OrderStatus.processing,
        ):
            return "Placed " + template
        return self.id

    @classmethod
    def _manipulate_stripe_object_hook(cls, data):
        data["payment_intent"] = data["payment"]["payment_intent"]
        return data

    def _attach_objects_post_save_hook(
        self,
        cls,
        data,
        api_key=djstripe_settings.STRIPE_SECRET_KEY,
        pending_relations=None,
    ):
        super()._attach_objects_post_save_hook(
            cls, data, api_key=api_key, pending_relations=pending_relations
        )

        # sync every discount
        for discount in self.discounts:
            Discount.sync_from_stripe_data(discount, api_key=api_key)

    def cancel(self, api_key=None, stripe_account=None, **kwargs):
        """
        Cancels the order as well as the payment intent if one is attached.

        :param api_key: The api key to use for this request. \
            Defaults to djstripe_settings.STRIPE_SECRET_KEY.
        :type api_key: string
        :param stripe_account: The optional connected account \
            for which this request is being made.
        :type stripe_account: string
        """
        api_key = api_key or self.default_api_key
        # Prefer passed in stripe_account if set.
        if not stripe_account:
            stripe_account = self._get_stripe_account_id(api_key)

        return self.stripe_class.cancel(
            self.id,
            api_key=api_key,
            stripe_account=stripe_account,
            stripe_version=djstripe_settings.STRIPE_API_VERSION,
            **kwargs,
        )

    def reopen(self, api_key=None, stripe_account=None, **kwargs):
        """
        Reopens a submitted order.

        :param api_key: The api key to use for this request. \
            Defaults to djstripe_settings.STRIPE_SECRET_KEY.
        :type api_key: string
        :param stripe_account: The optional connected account \
            for which this request is being made.
        :type stripe_account: string
        """
        api_key = api_key or self.default_api_key
        # Prefer passed in stripe_account if set.
        if not stripe_account:
            stripe_account = self._get_stripe_account_id(api_key)

        return self.stripe_class.reopen(
            self.id,
            api_key=api_key,
            stripe_account=stripe_account,
            stripe_version=djstripe_settings.STRIPE_API_VERSION,
            **kwargs,
        )

    def submit(self, api_key=None, stripe_account=None, **kwargs):
        """
        Submitting an Order transitions the status to processing and creates a PaymentIntent object
        so the order can be paid.
        If the Order has an amount_total of 0, no PaymentIntent object will be created.
        Once the order is submitted, its contents cannot be changed,
        unless the reopen method is called.

        :param api_key: The api key to use for this request. \
            Defaults to djstripe_settings.STRIPE_SECRET_KEY.
        :type api_key: string
        :param stripe_account: The optional connected account \
            for which this request is being made.
        :type stripe_account: string
        """
        api_key = api_key or self.default_api_key
        # Prefer passed in stripe_account if set.
        if not stripe_account:
            stripe_account = self._get_stripe_account_id(api_key)

        return self.stripe_class.submit(
            self.id,
            api_key=api_key,
            stripe_account=stripe_account,
            stripe_version=djstripe_settings.STRIPE_API_VERSION,
            **kwargs,
        )



"""
dj-stripe System Checks
"""
import re

from django.core import checks
from django.db.utils import DatabaseError

STRIPE_API_VERSION_PATTERN = re.compile(
    r"(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})(; [\w=]*)?$"
)


# 4 possibilities:
# Keys in admin and in settings
# Keys in admin and not in settings
# Keys not in admin but in settings
# Keys not in admin and not in settings
@checks.register("djstripe")
def check_stripe_api_key(app_configs=None, **kwargs):
    """Check the user has configured API live/test keys correctly."""

    def _check_stripe_api_in_settings(messages):
        if djstripe_settings.STRIPE_LIVE_MODE:
            if not djstripe_settings.LIVE_API_KEY.startswith(("sk_live_", "rk_live_")):
                msg = "Bad Stripe live API key."
                hint = 'STRIPE_LIVE_SECRET_KEY should start with "sk_live_"'
                messages.append(checks.Info(msg, hint=hint, id="djstripe.I003"))
        elif not djstripe_settings.TEST_API_KEY.startswith(("sk_test_", "rk_test_")):
            msg = "Bad Stripe test API key."
            hint = 'STRIPE_TEST_SECRET_KEY should start with "sk_test_"'
            messages.append(checks.Info(msg, hint=hint, id="djstripe.I004"))

    from djstripe.models import APIKey

    from .settings import djstripe_settings

    messages = []

    try:
        # get all APIKey objects in the db
        api_qs = APIKey.objects.all()

        if not api_qs.exists():
            msg = (
                "You don't have any API Keys in the database. Did you forget to add"
                " them?"
            )
            hint = (
                "Add STRIPE_TEST_SECRET_KEY and STRIPE_LIVE_SECRET_KEY directly from"
                " the Django Admin."
            )
            messages.append(checks.Info(msg, hint=hint, id="djstripe.I001"))

            # Keys not in admin but in settings
            if djstripe_settings.STRIPE_SECRET_KEY:
                msg = (
                    "Your keys are defined in the settings files. You can now add and"
                    " manage them directly from the django admin."
                )
                hint = (
                    "Add STRIPE_TEST_SECRET_KEY and STRIPE_LIVE_SECRET_KEY directly"
                    " from the Django Admin."
                )
                messages.append(checks.Info(msg, hint=hint, id="djstripe.I002"))

                # Ensure keys defined in settings files are valid
                _check_stripe_api_in_settings(messages)

        # Keys in admin and in settings
        elif djstripe_settings.STRIPE_SECRET_KEY:
            msg = (
                "Your keys are defined in the settings files and are also in the admin."
                " You can now add and manage them directly from the django admin."
            )
            hint = (
                "We suggest adding STRIPE_TEST_SECRET_KEY and STRIPE_LIVE_SECRET_KEY"
                " directly from the Django Admin. And removing them from the settings"
                " files."
            )
            messages.append(checks.Info(msg, hint=hint, id="djstripe.I002"))

            # Ensure keys defined in settings files are valid
            _check_stripe_api_in_settings(messages)

    except DatabaseError:
        # Skip the check - Database most likely not migrated yet
        return []

    return messages


def validate_stripe_api_version(version):
    """
    Check the API version is formatted correctly for Stripe.

    The expected format is `YYYY-MM-DD` (an iso8601 date) or
    for access to alpha or beta releases the expected format is: `YYYY-MM-DD; modelname_version=version_number`.
    Ex "2020-08-27; orders_beta=v3"

    :param version: The version to set for the Stripe API.
    :type version: ``str``
    :returns bool: Whether the version is formatted correctly.
    """
    return re.match(STRIPE_API_VERSION_PATTERN, version)


@checks.register("djstripe")
def check_stripe_api_version(app_configs=None, **kwargs):
    """Check the user has configured API version correctly."""
    from .settings import djstripe_settings

    messages = []
    default_version = djstripe_settings.DEFAULT_STRIPE_API_VERSION
    version = djstripe_settings.STRIPE_API_VERSION

    if not validate_stripe_api_version(version):
        msg = f"Invalid Stripe API version: {version!r}"
        hint = "STRIPE_API_VERSION should be formatted as: YYYY-MM-DD"
        messages.append(checks.Critical(msg, hint=hint, id="djstripe.C004"))

    if version != default_version:
        msg = (
            f"The Stripe API version has a non-default value of '{version!r}'. "
            "Non-default versions are not explicitly supported, and may "
            "cause compatibility issues."
        )
        hint = f"Use the dj-stripe default for Stripe API version: {default_version}"
        messages.append(checks.Warning(msg, hint=hint, id="djstripe.W001"))

    return messages


@checks.register("djstripe")
def check_stripe_api_host(app_configs=None, **kwargs):
    """
    Check that STRIPE_API_HOST is not being used in production.
    """
    from django.conf import settings

    messages = []

    if not settings.DEBUG and hasattr(settings, "STRIPE_API_HOST"):
        messages.append(
            checks.Warning(
                (
                    "STRIPE_API_HOST should not be set in production! "
                    "This is most likely unintended."
                ),
                hint="Remove STRIPE_API_HOST from your Django settings.",
                id="djstripe.W002",
            )
        )

    return messages


@checks.register("djstripe")
def check_webhook_secret(app_configs=None, **kwargs):
    """
    Check that DJSTRIPE_WEBHOOK_SECRET looks correct
    """

    def check_webhook_endpoint_secret(secret, messages, endpoint=None):
        if secret and not secret.startswith("whsec_"):
            if endpoint:
                extra_msg = (
                    f"The secret for Webhook Endpoint: {endpoint} does not look valid"
                )
            else:
                extra_msg = "DJSTRIPE_WEBHOOK_SECRET does not look valid"

            messages.append(
                checks.Warning(
                    extra_msg,
                    hint="It should start with whsec_...",
                    id="djstripe.W003",
                )
            )
        return messages

    from .models import WebhookEndpoint
    from .settings import djstripe_settings

    messages = []
    try:
        webhooks = list(WebhookEndpoint.objects.all())
    except DatabaseError:
        # skip the db-based check (db most likely not migrated yet)
        webhooks = []

    if webhooks:
        for endpoint in webhooks:
            secret = endpoint.secret
            # check secret
            check_webhook_endpoint_secret(secret, messages, endpoint=endpoint)
    else:
        secret = djstripe_settings.WEBHOOK_SECRET
        # check secret
        check_webhook_endpoint_secret(secret, messages)

    return messages


def _check_webhook_endpoint_validation(secret, messages, endpoint=None):
    if not secret:
        if endpoint:
            extra_msg = f"but Webhook Endpoint: {endpoint} has no secret set"
            secret_attr = "secret"
        else:
            extra_msg = "but DJSTRIPE_WEBHOOK_SECRET is not set"
            secret_attr = "DJSTRIPE_WEBHOOK_SECRET"

        messages.append(
            checks.Info(
                f"DJSTRIPE_WEBHOOK_VALIDATION is set to 'verify_signature' {extra_msg}",
                hint=(
                    f"Set {secret_attr} from Django shell or set"
                    " DJSTRIPE_WEBHOOK_VALIDATION='retrieve_event'"
                ),
                id="djstripe.I006",
            )
        )
    return messages


@checks.register("djstripe")
def check_webhook_validation(app_configs=None, **kwargs):
    """
    Check that DJSTRIPE_WEBHOOK_VALIDATION is valid
    """
    from .models import WebhookEndpoint
    from .settings import djstripe_settings

    setting_name = "DJSTRIPE_WEBHOOK_VALIDATION"

    messages = []

    validation_options = ("verify_signature", "retrieve_event")

    if djstripe_settings.WEBHOOK_VALIDATION is None:
        messages.append(
            checks.Warning(
                (
                    "Webhook validation is disabled, this is a security risk if the "
                    "webhook view is enabled"
                ),
                hint=f"Set {setting_name} to one of: {validation_options}",
                id="djstripe.W004",
            )
        )
    elif djstripe_settings.WEBHOOK_VALIDATION == "verify_signature":
        try:
            webhooks = list(WebhookEndpoint.objects.all())
        except DatabaseError:
            # Skip the db-based check (database most likely not migrated yet)
            webhooks = []

        if webhooks:
            for endpoint in webhooks:
                secret = endpoint.secret
                # check secret
                _check_webhook_endpoint_validation(secret, messages, endpoint=endpoint)
        else:
            secret = djstripe_settings.WEBHOOK_SECRET
            # check secret
            _check_webhook_endpoint_validation(secret, messages)

    elif djstripe_settings.WEBHOOK_VALIDATION not in validation_options:
        messages.append(
            checks.Critical(
                f"{setting_name} is invalid",
                hint=f"Set {setting_name} to one of: {validation_options} or None",
                id="djstripe.C007",
            )
        )

    return messages


@checks.register("djstripe")
def check_webhook_endpoint_has_secret(app_configs=None, **kwargs):
    """Checks if all Webhook Endpoints have not empty secrets."""
    from djstripe.models import WebhookEndpoint

    messages = []

    try:
        qs = list(WebhookEndpoint.objects.filter(secret="").all())
    except DatabaseError:
        # Skip the check - Database most likely not migrated yet
        return []

    for webhook in qs:
        webhook_url = webhook.get_stripe_dashboard_url()
        messages.append(
            checks.Warning(
                (
                    f"The secret of Webhook Endpoint: {webhook} is not populated "
                    "in the db. Events sent to it will not work properly."
                ),
                hint=(
                    "This can happen if it was deleted and resynced as Stripe "
                    "sends the webhook secret ONLY on the creation call. "
                    "Please use the django shell and update the secret with "
                    f"the value from {webhook_url}"
                ),
                id="djstripe.W005",
            )
        )

    return messages


@checks.register("djstripe")
def check_subscriber_key_length(app_configs=None, **kwargs):
    """
    Check that DJSTRIPE_SUBSCRIBER_CUSTOMER_KEY fits in metadata.

    Docs: https://stripe.com/docs/api#metadata
    """
    from .settings import djstripe_settings

    messages = []

    key = djstripe_settings.SUBSCRIBER_CUSTOMER_KEY
    key_max_length = 40
    if key and len(key) > key_max_length:
        messages.append(
            checks.Error(
                (
                    "DJSTRIPE_SUBSCRIBER_CUSTOMER_KEY must be no more than "
                    f"{key_max_length} characters long"
                ),
                hint=f"Current value: {key!r}",
                id="djstripe.E001",
            )
        )

    return messages


@checks.register("djstripe")
def check_djstripe_settings_foreign_key_to_field(app_configs=None, **kwargs):
    """
    Check that DJSTRIPE_FOREIGN_KEY_TO_FIELD is set to a valid value.
    """
    from django.conf import settings

    setting_name = "DJSTRIPE_FOREIGN_KEY_TO_FIELD"
    hint = (
        f'Set {setting_name} to "id" if this is a new installation, '
        'otherwise set it to "djstripe_id".'
    )
    messages = []

    if not hasattr(settings, setting_name):
        messages.append(
            checks.Error(
                f"{setting_name} is not set.",
                hint=hint,
                id="djstripe.E002",
            )
        )
    elif getattr(settings, setting_name) not in ("id", "djstripe_id"):
        setting_value = getattr(settings, setting_name)
        messages.append(
            checks.Error(
                f"{setting_value} is not a valid value for {setting_name}.",
                hint=hint,
                id="djstripe.E003",
            )
        )

    return messages


@checks.register("djstripe")
def check_webhook_event_callback_accepts_api_key(app_configs=None, **kwargs):
    """
    Checks if the custom callback accepts atleast 2 mandatory positional arguments
    """
    from inspect import signature

    from .settings import djstripe_settings

    messages = []

    # callable can have exactly 2 arguments or
    # if more than two, the rest need to be optional.
    callable = djstripe_settings.WEBHOOK_EVENT_CALLBACK

    if callable:
        # Deprecated in 2.8.0. Raise a warning.
        messages.append(
            checks.Warning(
                (
                    "DJSTRIPE_WEBHOOK_EVENT_CALLBACK is deprecated. See release notes"
                    " for details."
                ),
                hint=(
                    "If you need to trigger a function during webhook processing, "
                    "you can use djstripe.signals instead.\n"
                    "Available signals:\n"
                    "- djstripe.signals.webhook_pre_validate\n"
                    "- djstripe.signals.webhook_post_validate\n"
                    "- djstripe.signals.webhook_pre_process\n"
                    "- djstripe.signals.webhook_post_process\n"
                    "- djstripe.signals.webhook_processing_error"
                ),
            )
        )

        sig = signature(callable)
        signature_sz = len(sig.parameters.keys())

        if signature_sz < 2:
            messages.append(
                checks.Error(
                    f"{callable} accepts {signature_sz} arguments.",
                    hint=(
                        "You may have forgotten to add api_key parameter to your custom"
                        " callback."
                    ),
                    id="djstripe.E004",
                )
            )

    return messages



# Generated by Django 4.2 on 2023-04-25 00:50

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import djstripe.enums
import djstripe.fields


class Migration(migrations.Migration):
    dependencies = [
        ("djstripe", "0011_2_7"),
    ]

    operations = [
        migrations.CreateModel(
            name="InvoiceOrLineItem",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                (
                    "type",
                    djstripe.fields.StripeEnumField(
                        enum=djstripe.enums.InvoiceorLineItemType, max_length=12
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="VerificationReport",
            fields=[
                ("djstripe_created", models.DateTimeField(auto_now_add=True)),
                ("djstripe_updated", models.DateTimeField(auto_now=True)),
                (
                    "djstripe_id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("id", djstripe.fields.StripeIdField(max_length=255, unique=True)),
                (
                    "livemode",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text=(
                            "Null here indicates that the livemode status is unknown or"
                            " was previously unrecorded. Otherwise, this field"
                            " indicates whether this record comes from Stripe test mode"
                            " or live mode operation."
                        ),
                        null=True,
                    ),
                ),
                ("created", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                ("metadata", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "description",
                    models.TextField(
                        blank=True, help_text="A description of this object.", null=True
                    ),
                ),
                ("document", djstripe.fields.JSONField(blank=True, null=True)),
                ("id_number", djstripe.fields.JSONField(blank=True, null=True)),
                ("options", djstripe.fields.JSONField(blank=True, null=True)),
                ("selfie", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "type",
                    djstripe.fields.StripeEnumField(
                        enum=djstripe.enums.VerificationType, max_length=9
                    ),
                ),
                (
                    "djstripe_owner_account",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text="The Stripe Account this object belongs to.",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.account",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
            ],
            options={
                "get_latest_by": "created",
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="customer",
            name="discount",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="invoice",
            name="discounts",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="invoiceitem",
            name="discounts",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="affirm",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="blik",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="customer_balance",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="klarna",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="konbini",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="link",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="paynow",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="pix",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="promptpay",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="paymentmethod",
            name="us_bank_account",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="payout",
            name="original_payout",
            field=models.OneToOneField(
                blank=True,
                help_text=(
                    "If this payout reverses another, this is the ID of the original"
                    " payout."
                ),
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="djstripe.payout",
            ),
        ),
        migrations.AddField(
            model_name="payout",
            name="reversed_by",
            field=models.OneToOneField(
                blank=True,
                help_text=(
                    "If this payout was reversed, this is the ID of the payout that"
                    " reverses this payout."
                ),
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="reversed_payout",
                to="djstripe.payout",
            ),
        ),
        migrations.AddField(
            model_name="product",
            name="default_price",
            field=djstripe.fields.StripeForeignKey(
                blank=True,
                help_text="The default price this product is associated with.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="products",
                to="djstripe.price",
                to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
            ),
        ),
        migrations.AddField(
            model_name="session",
            name="amount_subtotal",
            field=djstripe.fields.StripeQuantumCurrencyAmountField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="session",
            name="amount_total",
            field=djstripe.fields.StripeQuantumCurrencyAmountField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="session",
            name="currency",
            field=djstripe.fields.StripeCurrencyCodeField(
                blank=True, max_length=3, null=True
            ),
        ),
        migrations.AddField(
            model_name="session",
            name="line_items",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="session",
            name="payment_status",
            field=djstripe.fields.StripeEnumField(
                blank=True,
                enum=djstripe.enums.SessionPaymentStatus,
                max_length=19,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="session",
            name="setup_intent",
            field=djstripe.fields.StripeForeignKey(
                blank=True,
                help_text=(
                    "The ID of the SetupIntent for Checkout Sessions in setup mode."
                ),
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="djstripe.setupintent",
                to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
            ),
        ),
        migrations.AddField(
            model_name="session",
            name="shipping_address_collection",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="session",
            name="shipping_cost",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="session",
            name="shipping_details",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="session",
            name="shipping_options",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="session",
            name="status",
            field=djstripe.fields.StripeEnumField(
                blank=True, enum=djstripe.enums.SessionStatus, max_length=8, null=True
            ),
        ),
        migrations.AddField(
            model_name="session",
            name="total_details",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="session",
            name="url",
            field=models.TextField(
                blank=True,
                help_text=(
                    "The URL to the Checkout Session. Redirect customers to this URL to"
                    " take them to Checkout. If you’re using Custom Domains, the URL"
                    " will use your subdomain. Otherwise, it’ll use"
                    " checkout.stripe.com. This value is only present when the session"
                    " is active."
                ),
                max_length=5000,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="upcominginvoice",
            name="discounts",
            field=djstripe.fields.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="webhookendpoint",
            name="tolerance",
            field=models.PositiveSmallIntegerField(
                default=300,
                help_text="Controls the milliseconds tolerance which wards against replay attacks. Leave this to its default value unless you know what you're doing.",
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="business_type",
            field=djstripe.fields.StripeEnumField(
                blank=True, default="", enum=djstripe.enums.BusinessType, max_length=17
            ),
        ),
        migrations.CreateModel(
            name="VerificationSession",
            fields=[
                ("djstripe_created", models.DateTimeField(auto_now_add=True)),
                ("djstripe_updated", models.DateTimeField(auto_now=True)),
                (
                    "djstripe_id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("id", djstripe.fields.StripeIdField(max_length=255, unique=True)),
                (
                    "livemode",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text=(
                            "Null here indicates that the livemode status is unknown or"
                            " was previously unrecorded. Otherwise, this field"
                            " indicates whether this record comes from Stripe test mode"
                            " or live mode operation."
                        ),
                        null=True,
                    ),
                ),
                ("created", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                ("metadata", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "description",
                    models.TextField(
                        blank=True, help_text="A description of this object.", null=True
                    ),
                ),
                ("last_error", djstripe.fields.JSONField(blank=True, null=True)),
                ("redaction", djstripe.fields.JSONField(blank=True, null=True)),
                ("verified_outputs", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "status",
                    djstripe.fields.StripeEnumField(
                        enum=djstripe.enums.VerificationSessionStatus, max_length=14
                    ),
                ),
                (
                    "type",
                    djstripe.fields.StripeEnumField(
                        enum=djstripe.enums.VerificationType, max_length=9
                    ),
                ),
                (
                    "djstripe_owner_account",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text="The Stripe Account this object belongs to.",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.account",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "last_verification_report",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="djstripe.verificationreport",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
            ],
            options={
                "get_latest_by": "created",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="SourceTransaction",
            fields=[
                ("djstripe_created", models.DateTimeField(auto_now_add=True)),
                ("djstripe_updated", models.DateTimeField(auto_now=True)),
                (
                    "djstripe_id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("id", djstripe.fields.StripeIdField(max_length=255, unique=True)),
                (
                    "livemode",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text=(
                            "Null here indicates that the livemode status is unknown or"
                            " was previously unrecorded. Otherwise, this field"
                            " indicates whether this record comes from Stripe test mode"
                            " or live mode operation."
                        ),
                        null=True,
                    ),
                ),
                ("created", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                ("ach_credit_transfer", djstripe.fields.JSONField()),
                (
                    "amount",
                    djstripe.fields.StripeDecimalCurrencyAmountField(
                        blank=True, decimal_places=2, max_digits=11, null=True
                    ),
                ),
                ("currency", djstripe.fields.StripeCurrencyCodeField(max_length=3)),
                ("customer_data", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "status",
                    djstripe.fields.StripeEnumField(
                        enum=djstripe.enums.SourceTransactionStatus, max_length=10
                    ),
                ),
                (
                    "djstripe_owner_account",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text="The Stripe Account this object belongs to.",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.account",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "source",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="djstripe.source",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
            ],
            options={
                "get_latest_by": "created",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="LineItem",
            fields=[
                ("djstripe_created", models.DateTimeField(auto_now_add=True)),
                ("djstripe_updated", models.DateTimeField(auto_now=True)),
                (
                    "djstripe_id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("id", djstripe.fields.StripeIdField(max_length=255, unique=True)),
                (
                    "livemode",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text=(
                            "Null here indicates that the livemode status is unknown or"
                            " was previously unrecorded. Otherwise, this field"
                            " indicates whether this record comes from Stripe test mode"
                            " or live mode operation."
                        ),
                        null=True,
                    ),
                ),
                ("created", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                ("metadata", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "description",
                    models.TextField(
                        blank=True, help_text="A description of this object.", null=True
                    ),
                ),
                ("amount", djstripe.fields.StripeQuantumCurrencyAmountField()),
                (
                    "amount_excluding_tax",
                    djstripe.fields.StripeQuantumCurrencyAmountField(),
                ),
                ("currency", djstripe.fields.StripeCurrencyCodeField(max_length=3)),
                ("discount_amounts", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "discountable",
                    models.BooleanField(
                        default=False,
                        help_text=(
                            "If True, discounts will apply to this line item. Always"
                            " False for prorations."
                        ),
                    ),
                ),
                ("discounts", djstripe.fields.JSONField(blank=True, null=True)),
                ("period", djstripe.fields.JSONField()),
                ("period_end", djstripe.fields.StripeDateTimeField()),
                ("period_start", djstripe.fields.StripeDateTimeField()),
                ("price", djstripe.fields.JSONField()),
                (
                    "proration",
                    models.BooleanField(
                        default=False,
                        help_text=(
                            "Whether or not the invoice item was created automatically"
                            " as a proration adjustment when the customer switched"
                            " plans."
                        ),
                    ),
                ),
                ("proration_details", djstripe.fields.JSONField()),
                ("tax_amounts", djstripe.fields.JSONField(blank=True, null=True)),
                ("tax_rates", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "type",
                    djstripe.fields.StripeEnumField(
                        enum=djstripe.enums.LineItem, max_length=12
                    ),
                ),
                (
                    "unit_amount_excluding_tax",
                    djstripe.fields.StripeDecimalCurrencyAmountField(
                        blank=True, decimal_places=2, max_digits=11, null=True
                    ),
                ),
                (
                    "quantity",
                    models.IntegerField(
                        blank=True,
                        help_text=(
                            "The quantity of the subscription, if the line item is a"
                            " subscription or a proration."
                        ),
                        null=True,
                    ),
                ),
                (
                    "djstripe_owner_account",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text="The Stripe Account this object belongs to.",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.account",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "invoice_item",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text=(
                            "The ID of the invoice item associated with this line item"
                            " if any."
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.invoiceitem",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "subscription",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text=(
                            "The subscription that the invoice item pertains to, if"
                            " any."
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.subscription",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "subscription_item",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text=(
                            "The subscription item that generated this invoice item."
                            " Left empty if the line item is not an explicit result of"
                            " a subscription."
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.subscriptionitem",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
            ],
            options={
                "get_latest_by": "created",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Discount",
            fields=[
                ("djstripe_created", models.DateTimeField(auto_now_add=True)),
                ("djstripe_updated", models.DateTimeField(auto_now=True)),
                (
                    "djstripe_id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("id", djstripe.fields.StripeIdField(max_length=255, unique=True)),
                (
                    "livemode",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text=(
                            "Null here indicates that the livemode status is unknown or"
                            " was previously unrecorded. Otherwise, this field"
                            " indicates whether this record comes from Stripe test mode"
                            " or live mode operation."
                        ),
                        null=True,
                    ),
                ),
                ("created", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                ("metadata", djstripe.fields.JSONField(blank=True, null=True)),
                (
                    "description",
                    models.TextField(
                        blank=True, help_text="A description of this object.", null=True
                    ),
                ),
                ("coupon", djstripe.fields.JSONField(blank=True, null=True)),
                ("end", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                (
                    "promotion_code",
                    models.CharField(
                        blank=True,
                        help_text="The promotion code applied to create this discount.",
                        max_length=255,
                    ),
                ),
                ("start", djstripe.fields.StripeDateTimeField(blank=True, null=True)),
                (
                    "checkout_session",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text=(
                            "The Checkout session that this coupon is applied to, if it"
                            " is applied to a particular session in payment mode. Will"
                            " not be present for subscription mode."
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.session",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "customer",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text=(
                            "The ID of the customer associated with this discount."
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="customer_discounts",
                        to="djstripe.customer",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "djstripe_owner_account",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text="The Stripe Account this object belongs to.",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.account",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "invoice",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text=(
                            "The invoice that the discount’s coupon was applied to, if"
                            " it was applied directly to a particular invoice."
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="invoice_discounts",
                        to="djstripe.invoice",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
                (
                    "invoice_item",
                    djstripe.fields.InvoiceOrLineItemForeignKey(
                        blank=True,
                        help_text=(
                            "The invoice item id (or invoice line item id for invoice"
                            " line items of type=‘subscription’) that the discount’s"
                            " coupon was applied to, if it was applied directly to a"
                            " particular invoice item or invoice line item."
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="djstripe.invoiceorlineitem",
                    ),
                ),
                (
                    "subscription",
                    djstripe.fields.StripeForeignKey(
                        blank=True,
                        help_text=(
                            "The subscription that this coupon is applied to, if it is"
                            " applied to a particular subscription."
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="subscription_discounts",
                        to="djstripe.subscription",
                        to_field=settings.DJSTRIPE_FOREIGN_KEY_TO_FIELD,
                    ),
                ),
            ],
            options={
                "get_latest_by": "created",
                "abstract": False,
            },
        ),
    ]
